@@ -373,21 +373,31 @@ GdipClonePath (GpPath *path, GpPath **clonePath)
 	result->fill_mode = path->fill_mode;
 	result->count = path->count;
 	result->size = path->size;
-	result->points = GdipAlloc (sizeof (GpPointF) * result->size);
-	if (!result->points) {
-		GdipFree (result);
-		return OutOfMemory;
+
+	if (path->points) {
+		result->points = GdipAlloc (sizeof (GpPointF) * result->size);
+		if (!result->points) {
+			GdipFree (result);
+			return OutOfMemory;
+		}
+
+		memcpy (result->points, path->points, sizeof (GpPointF) * path->count);
+	} else {
+		result->points = NULL;
 	}
 
-	result->types = GdipAlloc (sizeof (BYTE) * result->size);
-	if (!result->types) {
-		GdipFree (result->points);
-		GdipFree (result);
-		return OutOfMemory;
-	}
+	if (path->types) {
+		result->types = GdipAlloc (sizeof (BYTE) * result->size);
+		if (!result->types) {
+			GdipFree (result->points);
+			GdipFree (result);
+			return OutOfMemory;
+		}
 
-	memcpy (result->points, path->points, sizeof (GpPointF) * path->count);
-	memcpy (result->types, path->types, sizeof (BYTE) * path->count);
+		memcpy (result->types, path->types, sizeof (BYTE) * path->count);
+	} else {
+		result->types = NULL;
+	}
 
 	result->start_new_fig = path->start_new_fig;
 
@@ -861,15 +871,12 @@ append_arcs (GpPath *path, float x, float y, float width, float height, float st
 }
 
 GpStatus WINGDIPAPI
-GdipAddPathArc (GpPath *path, float x, float y, 
-		float width, float height, float startAngle, float sweepAngle)
+GdipAddPathArc (GpPath *path, REAL x, REAL y, 
+		REAL width, REAL height, REAL startAngle, REAL sweepAngle)
 {
 	int point_count;
 
-	if (!path)
-		return InvalidParameter;
-
-	if (width == 0 || height == 0)
+	if (!path || width <= 0 || height <= 0)
 		return InvalidParameter;
 
 	point_count = count_arcs_points (path, x, y, width, height, startAngle, sweepAngle);
@@ -1109,13 +1116,13 @@ GdipAddPathEllipse (GpPath *path, float x, float y, float width, float height)
 }
 
 GpStatus WINGDIPAPI
-GdipAddPathPie (GpPath *path, float x, float y, float width, float height, float startAngle, float sweepAngle)
+GdipAddPathPie (GpPath *path, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
 {
 	int point_count;
 
 	float sin_alpha, cos_alpha;
 
-	if (width == 0 || height == 0)
+	if (!path || width <= 0 || height <= 0)
 		return InvalidParameter;
 
 	float rx = width / 2;
@@ -1134,9 +1141,6 @@ GdipAddPathPie (GpPath *path, float x, float y, float width, float height, float
 	sin_alpha = sin (alpha);
 	cos_alpha = cos (alpha);
 
-	if (!path)
-		return InvalidParameter;
-
 	point_count = count_arcs_points (path, x, y, width, height, startAngle, sweepAngle) + 1;
 	if (fabs (sweepAngle) < 360)
 		point_count += 2;
@@ -1425,7 +1429,7 @@ GdipAddPathLine2I (GpPath* path, const GpPoint *points, int count)
 }
 
 GpStatus WINGDIPAPI
-GdipAddPathArcI (GpPath *path, int x, int y, int width, int height, float startAngle, float sweepAngle)
+GdipAddPathArcI (GpPath *path, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
 {
 	return GdipAddPathArc (path, x, y, width, height, startAngle, sweepAngle);
 }
@@ -1565,7 +1569,7 @@ GdipAddPathEllipseI (GpPath *path, int x, int y, int width, int height)
 }
 
 GpStatus WINGDIPAPI
-GdipAddPathPieI (GpPath *path, int x, int y, int width, int height, float startAngle, float sweepAngle)
+GdipAddPathPieI (GpPath *path, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
 {
 	return GdipAddPathPie (path, x, y, width, height, startAngle, sweepAngle);
 }
