@@ -1,22 +1,22 @@
 /*
  * Copyright (C) 2004 Ximian inc. http://www.ximian.com
  * Copyright (C) 2004,2006-2007 Novell Inc. http://www.novell.com
- * 
- * Permission is hereby granted, free of charge, to any person obtaining a copy of this software 
- * and associated documentation files (the "Software"), to deal in the Software without restriction, 
- * including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
- * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
+ * and associated documentation files (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
  * subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included in all copies or substantial 
+ *
+ * The above copyright notice and this permission notice shall be included in all copies or substantial
  * portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
- * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
- * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
+ * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- * 
+ *
  * Authors:
  *	  Jordi Mas i Hernandez <jordi@ximian.com>, 2004
  *	  Ravindra <rkumar@novell.com>
@@ -25,113 +25,14 @@
  */
 
 #include "stringformat-private.h"
+#include "general-private.h"
 
-/* coverity[+alloc : arg-*2] */
-GpStatus WINGDIPAPI
-GdipCreateStringFormat (INT formatAttributes, LANGID language, GpStringFormat **format)
+static GpStringFormat stringFormatDefault;
+static GpStringFormat stringFormatTypographic;
+
+static void
+gdip_string_format_clear (GpStringFormat *format)
 {
-	GpStringFormat *result;
-
-	if (!format)
-		return InvalidParameter;
-
-	result = (GpStringFormat *) GdipAlloc (sizeof (GpStringFormat));
-
-	if (!result)
-		return OutOfMemory;
-
-	result->alignment = StringAlignmentNear;
-	result->lineAlignment =  StringAlignmentNear;
-	result->hotkeyPrefix = HotkeyPrefixNone;
-	result->formatFlags = formatAttributes; 
-	result->trimming = StringTrimmingCharacter;
-	result->substitute = StringDigitSubstituteUser;
-	result->firstTabOffset = 0;
-	result->tabStops = NULL;
-	result->numtabStops = 0;
-	result->charRanges = NULL;
-	result->charRangeCount = 0;
-
-	*format = result;
-	return Ok;
-}
-
-/* coverity[+alloc : arg-*0] */
-GpStatus WINGDIPAPI
-GdipStringFormatGetGenericDefault (GpStringFormat **format)
-{
-	return GdipCreateStringFormat (0, 0, format);
-}
-
-/* coverity[+alloc : arg-*0] */
-GpStatus WINGDIPAPI
-GdipStringFormatGetGenericTypographic (GpStringFormat **format)
-{
-	int formatFlags = StringFormatFlagsNoFitBlackBox | StringFormatFlagsLineLimit | StringFormatFlagsNoClip;
-	GpStatus status = GdipCreateStringFormat (formatFlags, 0, format);
-	if (status == Ok)
-		(*format)->trimming = StringTrimmingNone;
-	return status;
-}
-
-/* coverity[+alloc : arg-*1] */
-GpStatus WINGDIPAPI
-GdipCloneStringFormat (GDIPCONST GpStringFormat *format,  GpStringFormat **newFormat)
-{
-	int i;
-	GpStringFormat *result;
-
-	if (!format || !newFormat)
-		return InvalidParameter;
-
-	result = (GpStringFormat *) GdipAlloc (sizeof (GpStringFormat));
-
-	if (!result)
-		return OutOfMemory;
-
-	result->alignment = format->alignment;
-	result->lineAlignment =  format->lineAlignment;
-	result->hotkeyPrefix = format->hotkeyPrefix;
-	result->formatFlags = format->formatFlags; 
-	result->trimming = format->trimming;
-	result->substitute = format->substitute;
-	result->firstTabOffset = format->firstTabOffset;
-	result->numtabStops = format->numtabStops;
-	result->charRangeCount = format->charRangeCount;
-
-	/* Create a copy of tab stops for the clone */
-	result->tabStops = (float *) GdipAlloc (sizeof (float) * format->numtabStops);
-	if (result->tabStops == NULL) {
-		GdipFree (result);
-		return OutOfMemory;
-	}
-
-	for (i = 0; i < format->numtabStops; i++)
-		result->tabStops [i] = format->tabStops [i];
-
-	/* Create a copy of char ranges for the clone */
-	result->charRanges = (CharacterRange *) GdipAlloc (format->charRangeCount * sizeof (CharacterRange));
-	if (result->charRanges == NULL) {
-		GdipFree (result->tabStops);
-		GdipFree (result);
-		return OutOfMemory;
-	}
-
-	for (i = 0; i < format->charRangeCount; i++) {
-		result->charRanges [i].First = format->charRanges [i].First;
-		result->charRanges [i].Length = format->charRanges [i].Length;
-	}
-
-	*newFormat = result;
-	return Ok; 
-}
-
-GpStatus WINGDIPAPI
-GdipDeleteStringFormat (GpStringFormat *format)
-{
-	if (!format)
-		return InvalidParameter;
-
 	if (format->tabStops) {
 		GdipFree (format->tabStops);
 		format->tabStops = NULL;
@@ -141,6 +42,164 @@ GdipDeleteStringFormat (GpStringFormat *format)
 		GdipFree (format->charRanges);
 		format->charRanges = NULL;
 	}
+}
+
+static void
+gdip_string_format_init (GpStringFormat *result)
+{
+	result->alignment = StringAlignmentNear;
+	result->lineAlignment =  StringAlignmentNear;
+	result->hotkeyPrefix = HotkeyPrefixNone;
+	result->formatFlags = 0;
+	result->trimming = StringTrimmingCharacter;
+	result->substitute = StringDigitSubstituteUser;
+	result->language = 0;
+	result->firstTabOffset = 0;
+	result->tabStops = NULL;
+	result->numtabStops = 0;
+	result->charRanges = NULL;
+	result->charRangeCount = 0;
+}
+
+void
+gdip_create_generic_stringformats ()
+{
+	gdip_string_format_init (&stringFormatDefault);
+
+	gdip_string_format_init (&stringFormatTypographic);
+	stringFormatTypographic.formatFlags = StringFormatFlagsNoFitBlackBox | StringFormatFlagsLineLimit | StringFormatFlagsNoClip;
+	stringFormatTypographic.trimming = StringTrimmingNone;
+}
+
+void
+gdip_delete_generic_stringformats ()
+{
+	gdip_string_format_clear (&stringFormatDefault);
+	gdip_string_format_clear (&stringFormatTypographic);
+}
+
+static GpStringFormat *
+gdip_string_format_new ()
+{
+	GpStringFormat *result = GdipAlloc (sizeof (GpStringFormat));
+	if (result)
+		gdip_string_format_init (result);
+
+	return result;
+}
+
+/* coverity[+alloc : arg-*2] */
+GpStatus WINGDIPAPI
+GdipCreateStringFormat (INT formatAttributes, LANGID language, GpStringFormat **format)
+{
+	GpStringFormat *result;
+
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	if (!format)
+		return InvalidParameter;
+
+	result = gdip_string_format_new ();
+	if (!result) {
+		*format = NULL;
+		return OutOfMemory;
+	}
+
+	result->formatFlags = formatAttributes;
+
+	*format = result;
+	return Ok;
+}
+
+/* coverity[+alloc : arg-*0] */
+GpStatus WINGDIPAPI
+GdipStringFormatGetGenericDefault (GpStringFormat **format)
+{
+	if (!format)
+		return InvalidParameter;
+
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	*format = &stringFormatDefault;
+	return Ok;
+}
+
+/* coverity[+alloc : arg-*0] */
+GpStatus WINGDIPAPI
+GdipStringFormatGetGenericTypographic (GpStringFormat **format)
+{
+	if (!format)
+		return InvalidParameter;
+
+	if (!gdiplusInitialized)
+		return GdiplusNotInitialized;
+
+	*format = &stringFormatTypographic;
+	return Ok;
+}
+
+/* coverity[+alloc : arg-*1] */
+GpStatus WINGDIPAPI
+GdipCloneStringFormat (GDIPCONST GpStringFormat *format,  GpStringFormat **newFormat)
+{
+	GpStringFormat *result;
+
+	if (!format || !newFormat)
+		return InvalidParameter;
+
+	result = gdip_string_format_new ();
+	if (!result)
+		goto error;
+
+	result->alignment = format->alignment;
+	result->lineAlignment =  format->lineAlignment;
+	result->hotkeyPrefix = format->hotkeyPrefix;
+	result->formatFlags = format->formatFlags;
+	result->trimming = format->trimming;
+	result->substitute = format->substitute;
+	result->language = format->language;
+	result->firstTabOffset = format->firstTabOffset;
+	result->numtabStops = format->numtabStops;
+	result->charRangeCount = format->charRangeCount;
+
+	/* Create a copy of tab stops for the clone */
+	result->tabStops = (float *) GdipAlloc (sizeof (float) * format->numtabStops);
+	if (!result->tabStops)
+		goto error;
+
+	memcpy (result->tabStops, format->tabStops, format->numtabStops * sizeof (float));
+
+	/* Create a copy of char ranges for the clone */
+	result->charRanges = (CharacterRange *) GdipAlloc (format->charRangeCount * sizeof (CharacterRange));
+	if (result->charRanges == NULL)
+		goto error;
+
+	memcpy (result->charRanges, format->charRanges, format->charRangeCount * sizeof (CharacterRange));
+
+	*newFormat = result;
+	return Ok;
+
+error:
+	if (result)
+		GdipDeleteStringFormat (result);
+
+	*newFormat = NULL;
+	return OutOfMemory;
+}
+
+GpStatus WINGDIPAPI
+GdipDeleteStringFormat (GpStringFormat *format)
+{
+	if (!format)
+		return InvalidParameter;
+
+	// These are singletons cleared on shutdown.
+	if (format == &stringFormatDefault || format == &stringFormatTypographic)
+		return Ok;
+
+	gdip_string_format_clear (format);
 
 	GdipFree (format);
 	return Ok;
@@ -149,7 +208,7 @@ GdipDeleteStringFormat (GpStringFormat *format)
 GpStatus WINGDIPAPI
 GdipSetStringFormatAlign (GpStringFormat *format, StringAlignment align)
 {
-	if (!format)
+	if (!format || align < StringAlignmentNear || align > StringAlignmentFar)
 		return InvalidParameter;
 
 	format->alignment = align;
@@ -169,7 +228,7 @@ GdipGetStringFormatAlign (GDIPCONST GpStringFormat *format, StringAlignment *ali
 GpStatus WINGDIPAPI
 GdipSetStringFormatLineAlign (GpStringFormat *format, StringAlignment align)
 {
-	if (!format)
+	if (!format || align < StringAlignmentNear || align > StringAlignmentFar)
 		return InvalidParameter;
 
 	format->lineAlignment = align;
@@ -189,7 +248,7 @@ GdipGetStringFormatLineAlign (GDIPCONST GpStringFormat *format, StringAlignment
 GpStatus WINGDIPAPI
 GdipSetStringFormatHotkeyPrefix (GpStringFormat *format, INT hotkeyPrefix)
 {
-	if (!format)
+	if (!format || hotkeyPrefix < HotkeyPrefixNone || hotkeyPrefix > HotkeyPrefixHide)
 		return InvalidParameter;
 
 	format->hotkeyPrefix = hotkeyPrefix;
@@ -229,7 +288,7 @@ GdipGetStringFormatFlags (GDIPCONST GpStringFormat *format, INT *flags)
 GpStatus WINGDIPAPI
 GdipSetStringFormatTrimming (GpStringFormat *format,  StringTrimming trimming)
 {
-	if (!format)
+	if (!format || trimming < StringTrimmingNone || trimming > StringTrimmingEllipsisPath)
 		return InvalidParameter;
 
 	format->trimming = trimming;
@@ -249,43 +308,53 @@ GdipGetStringFormatTrimming (GDIPCONST GpStringFormat *format, StringTrimming *t
 GpStatus WINGDIPAPI
 GdipSetStringFormatTabStops (GpStringFormat *format, REAL firstTabOffset, INT count, GDIPCONST REAL *tabStops)
 {
-	int i;
-	float *pItemSrc = (float*) tabStops;
-	float *pItemTrg;
+	REAL *tabStopsResult;
 
 	if (!format || !tabStops)
 		return InvalidParameter;
 
-	if (format->tabStops)
-		GdipFree (format->tabStops);
-
-	format->firstTabOffset = (count >= 0) ? firstTabOffset : 0.0f;
-
-	if (count <= 0) {
-		format->tabStops = NULL;
-		format->numtabStops = 0;
+	if (count <= 0)
 		return Ok;
+
+	if (firstTabOffset < 0)
+		return NotImplemented;
+
+	for (int i = 0; i < count; i++) {
+		if (tabStops[i] < 0)
+			return NotImplemented;
 	}
 
-	pItemTrg = format->tabStops = GdipAlloc (sizeof(float) * count);
+	/* Avoid allocating memory if the count is the same. */
+	if (count != format->numtabStops) {
+		tabStopsResult = GdipAlloc (count * sizeof (REAL));
+		if (!tabStopsResult)
+			return OutOfMemory;
 
-	if (!pItemTrg)
-		return OutOfMemory;
+		if (format->tabStops)
+			GdipFree (format->tabStops);
 
-	for (i = 0; i < count; i++, pItemSrc++, pItemTrg++)
-		*pItemTrg = *pItemSrc;
+		format->tabStops = tabStopsResult;
+	}
 
+	format->firstTabOffset = firstTabOffset;
 	format->numtabStops = count;
+	memcpy (format->tabStops, tabStops, count * sizeof (REAL));
+
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
 GdipGetStringFormatDigitSubstitution (GDIPCONST GpStringFormat *format, LANGID *language, StringDigitSubstitute *substitute)
 {
-	if (!format || !substitute)
+	if (!format)
 		return InvalidParameter;
 
-	*substitute = format->substitute;
+	if (language)
+		*language = format->language;
+
+	if (substitute)
+		*substitute = format->substitute;
+
 	return Ok;
 }
 
@@ -295,6 +364,7 @@ GdipSetStringFormatDigitSubstitution (GpStringFormat *format, LANGID language, S
 	if (!format)
 		return InvalidParameter;
 
+	format->language = language;
 	format->substitute = substitute;
 	return Ok;
 }
@@ -306,27 +376,21 @@ GdipGetStringFormatTabStopCount (GDIPCONST GpStringFormat *format, INT *count)
 		return InvalidParameter;
 
 	*count = format->numtabStops;
-
 	return Ok;
 }
 
 GpStatus WINGDIPAPI
 GdipGetStringFormatTabStops (GDIPCONST GpStringFormat *format, INT count, REAL *firstTabOffset, REAL *tabStops)
 {
-	int i;
-	float *pItemSrc;
-	float *pItemTrg = tabStops;
 	int elems;
 
-	if (!format || !firstTabOffset || !tabStops)
+	if (!format || !firstTabOffset || !tabStops || count < 0)
 		return InvalidParameter;
 
-	pItemSrc = format->tabStops;
 	elems = count<format->numtabStops ? count : format->numtabStops;
-	for (i = 0; i < elems; i++, pItemSrc++, pItemTrg++)
-		*pItemTrg = *pItemSrc;
-		
+
 	*firstTabOffset = format->firstTabOffset;
+	memcpy (tabStops, format->tabStops, elems * sizeof (REAL));
 
 	return Ok;
 }
@@ -344,26 +408,38 @@ GdipGetStringFormatMeasurableCharacterRangeCount (GDIPCONST GpStringFormat *form
 GpStatus WINGDIPAPI
 GdipSetStringFormatMeasurableCharacterRanges (GpStringFormat *format, INT rangeCount, GDIPCONST CharacterRange *ranges)
 {
-	if (!format || !ranges || (rangeCount < 0))
+	CharacterRange *rangesResult = NULL;
+
+	if (!format || !ranges)
 		return InvalidParameter;
 
-	if (rangeCount == 0)
-		return Ok;
-	else if (rangeCount > 32)
+	if (rangeCount > 32)
 		return ValueOverflow;
 
+	if (rangeCount > 0) {
+		if (format->charRangeCount == rangeCount) {
+			/* Avoid allocating memory if the count is the same. */
+			rangesResult = format->charRanges;
+		}
+		else {
+			rangesResult = (CharacterRange *) GdipAlloc (rangeCount * sizeof (CharacterRange));
+			if (!rangesResult)
+				return OutOfMemory;
+		}
+
+		memcpy (rangesResult, ranges, rangeCount * sizeof (CharacterRange));
+	}
+	else {
+		rangesResult = NULL;
+		rangeCount = 0;
+	}
+
 	if (format->charRanges && format->charRangeCount != rangeCount) {
 		GdipFree (format->charRanges);
 		format->charRanges = NULL;
 	}
 
-	if (format->charRanges == NULL) {
-		format->charRanges = (CharacterRange *) GdipAlloc (rangeCount * sizeof (CharacterRange));
-		if (format->charRanges == NULL)
-			return OutOfMemory;
-	}
-
-	memcpy (format->charRanges, ranges, rangeCount * sizeof (CharacterRange));
+	format->charRanges = rangesResult;
 	format->charRangeCount = rangeCount;
 
 	return Ok;
